package envoy.client.ui;

import java.awt.Color;

/**
 * This class stores the colors that are used in Envoy.
 * <br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>EnvoyColors.java</strong><br>
 * Created: <strong>16 Nov 2019</strong><br>
 * 
 * @author Leon Hofmeister
 * @since Envoy v0.2-alpha
 */
public class UIColors {

	private UIColors() {}

	/**
	 * This color is used for the general background where no other elements
	 * overlap.
	 * 
	 * @since Envoy v0.2-alpha
	 */
	private Color backgroundColor;

	/**
	 * This color is used as background for all areas where a user can interact with
	 * Envoy.
	 * (i.e. a JTextArea or JList)
	 * 
	 * @since Envoy v0.2-alpha
	 */

	private Color userInteractionColor;

	/**
	 * This color is used for any areas that need special attention.
	 * (i.e. highlighting a selected list column or a button background)
	 * 
	 * @since Envoy v0.2-alpha
	 */
	private Color specialUseColor;

	/**
	 * This color is the color in which text will be displayed.
	 * 
	 * @since Envoy v0.2-alpha
	 */
	private Color textColor;

	/**
	 * This object ensures that only one {@link UIColors} object exists.
	 * 
	 * @since Envoy v0.2-alpha
	 */
	private static UIColors uIColors;

	/**
	 * This method is used to ensure that there is only one instance of EnvoyColors.
	 * 
	 * @param darkMode default value how envoyColors should be displayed
	 * @return the instance of EnvoyColors
	 * @since Envoy v0.2-alpha
	 */
	public static UIColors getInstance(boolean darkMode) {
		if (uIColors == null) { uIColors = new UIColors(); }
		uIColors.setDisplayMode(darkMode);
		return uIColors;
	}

	/**
	 * Used to change the appearance of Envoy.
	 * 
	 * @param darkMode if true, Envoy will be displayed in dark mode else it will
	 *                 use bright mode
	 * @since Envoy v0.2-alpha
	 */
	public void setDisplayMode(boolean darkMode) {
		if (darkMode) {
			uIColors.setBackgroundColor(Color.black); // TODO: other color suggestions?
			uIColors.setUserInteractionColor(Color.darkGray); // temporary
			uIColors.setSpecialUseColor(Color.blue); // temporary
			uIColors.setTextColor(Color.white); // temporary

		} else {
			uIColors.setBackgroundColor(Color.white); // temporary
			uIColors.setUserInteractionColor(Color.lightGray); // temporary
			uIColors.setSpecialUseColor(Color.green); // temporary
			uIColors.setTextColor(Color.black); // temporary

		}
	}

	/**
	 * @return the {@link UIColors#backgroundColor}
	 * @since Envoy v0.2-alpha
	 */
	public Color getBackgroundColor() { return backgroundColor; }

	/**
	 * @param backgroundColor the new {@link UIColors#backgroundColor}
	 * @since Envoy v0.2-alpha
	 */
	public void setBackgroundColor(Color backgroundColor) { this.backgroundColor = backgroundColor; }

	/**
	 * @return the {@link UIColors#userInteractionColor}
	 * @since Envoy v0.2-alpha
	 */
	public Color getUserInteractionColor() { return userInteractionColor; }

	/**
	 * @param userInteractionColor the new {@link UIColors#userInteractionColor}
	 * @since Envoy v0.2-alpha
	 */
	public void setUserInteractionColor(Color userInteractionColor) {
		this.userInteractionColor = userInteractionColor;
	}

	/**
	 * @return the {@link UIColors#specialUseColor}
	 * @since Envoy v0.2-alpha
	 */
	public Color getSpecialUseColor() { return specialUseColor; }

	/**
	 * @param specialUseColor the new {@link UIColors#specialUseColor}
	 * @since Envoy v0.2-alpha
	 */
	public void setSpecialUseColor(Color specialUseColor) { this.specialUseColor = specialUseColor; }

	/**
	 * @return the {@link UIColors#textColor}
	 * @since Envoy v0.2-alpha
	 */
	public Color getTextColor() { return textColor; }

	/**
	 * @param textColor the new {@link UIColors#textColor}
	 * @since Envoy v0.2-alpha
	 */
	public void setTextColor(Color textColor) { this.textColor = textColor; }

	/**
	 * @return the {@link UIColors#uIColors}
	 * @since Envoy v0.2-alpha
	 */
	public static UIColors getEnvoyColors() { return uIColors; }

	@Deprecated
	/**
	 * @param envoyColors the new {@link EnvoyColors#envoyColors}
	 * @since Envoy v0.2-alpha
	 */
	public static void setEnvoyColors(UIColors uIColors) { UIColors.uIColors = uIColors; }
}